module Speedflow
  # Used to manage the commands
  class Command
    class << self
      include Message

      # @return [Hash] List of arguments
      attr_writer :argv

      # A list of subclasses of Speedflow::Command
      #
      # Returns Array of commands.
      def subclasses
        @subclasses ||= []
      end

      # Keep a list of subclasses of Speedflow::Command every time it's
      # inherited. Called automatically.
      #
      # base - the subclass
      #
      # Returns nothing.
      def inherited(base)
        subclasses << base
        super(base)
      end

      # Speedflow configuration with the options passed in as overrides
      #
      # options - the configuration overrides
      #
      # Returns a full Speedflow configuration.
      def config_from_options(options)
        Speedflow::Configuration.get(options)
      rescue ConfigurationFileNotFound => exception
        error exception.message
        abort
      rescue ConfigurationFileFormat => exception
        error exception.message
        abort
      end

      # Public: Get config from fresh options
      #
      # program    - Program.
      # command    - Commad.
      # options    - Options.
      #
      # Returns configuration Hash.
      def config_from_fresh_options(program, command, options = {})
        execute_from_option(program, command, 'config') do |value|
          options = { config: value }
        end

        config_from_options(options)
      end

      # Public: Exectute option from key
      #
      # prog    - Program.
      # command - Commad.
      # opt_key - Option key.
      #
      # Returns nothing.
      def execute_from_option(prog, command, opt_key)
        opt_index = prog.options.find_index { |o| o.config_key == opt_key }
        opt = prog.options[opt_index]
        opt_parser = OptionParser.new do |opts|
          command.add_default_options(opts)
          opts.on(*opt.for_option_parser) { |value| yield(value) }
        end

        # rubocop:disable HandleExceptions
        begin
          opt_parser.parse(argv)
        rescue OptionParser::InvalidOption
          # Nothing.
        end
        # rubocop:enable HandleExceptions
      end

      # Display invalid command.
      #
      # cmd  - the command
      #
      # Returns nothing
      def invalid_command(cmd)
        error "Command not found '#{cmd}'."
        notice 'Please, use --help to display valid commands.'
      end

      # Public: Define argv (used to test)
      #
      # Returns Array of argv.
      def argv
        @argv ||= ARGV
      end
    end
  end
end
