module Speedflow
  # Used to manage configuration in Speedflow core
  class Configuration < Hash
    # Default configuration filename
    DEFAULT_FILENAME = '.speedflow.yml'.freeze

    # Default options. Overridden by values in .speedflow.yml.
    DEFAULTS = Configuration[{
      version: 1,
      source:  Dir.pwd,
      plugins: [],
      flow:    {}
    }]

    # Public: Check triggers in flow
    #
    # Returns if flow has triggers.
    def flow_triggers?
      key?('flow') && !self['flow'].keys.empty?
    end

    # Public: Check trigger in flow
    #
    # trigger - Trigger name.
    #
    # Returns if trigger exists in flow.
    def flow_trigger?(trigger)
      key?('flow') && self['flow'].key?(trigger.to_s)
    end

    # Public: Directory of the Speedflow source folder
    #
    # override - the command-line options hash
    #
    # Returns the path to the Speedflow source directory.
    def source(override)
      get_config_value_with_override(:source, override)
    end

    # Public: Get config value with override
    #
    # config_key - Configuration key.
    # override   - An Hash of override values.
    #
    # Returns the value of config key or override.
    def get_config_value_with_override(config_key, override)
      override[config_key] || self[config_key] || DEFAULTS[config_key]
    end

    # Public: Generate list of configuration files from the override
    #
    # override - the command-line options hash
    #
    # Returns an Array of config files
    def config_files(override)
      files = []
      if override.stringify_keys.key?('config')
        files = override.stringify_keys.delete('config')
        override.delete('config')
      else
        files << source(override) + File::SEPARATOR + DEFAULT_FILENAME
      end
      files = [files] unless files.is_a? Array
      files
    end

    # Public: Read all files content
    #
    # files - the list of configuration file
    #
    # Returns the full configuration.
    def read_config_files(files)
      config = clone

      files.each do |file|
        unless File.exist?(file)
          raise ConfigurationFileNotFound, "Unable to load config file: #{file}"
        end

        config = config.deep_merge(read_config_file(file))
      end

      config.replace_values_from_env
    end

    # Public: Read config file
    #
    # file - File to read.
    #
    # Returns a Hash.
    def read_config_file(file)
      SafeYAML.load_file(file) || {}
    rescue
      raise ConfigurationFileFormat, "This file is not a valid YAML: #{file}"
    end

    # Get all configuration.
    #
    # override - Hash of configuration.
    #
    # Returns the full configuration.
    def self.get(override = {})
      config = self[self::DEFAULTS]
      override = self[override].stringify_keys

      config = config.read_config_files(config.config_files(override))
      config.deep_merge(override).stringify_keys

      config
    end
  end

  # File not found exception
  class ConfigurationFileNotFound < RuntimeError
  end

  # File format exception
  class ConfigurationFileFormat < RuntimeError
  end
end
