module Speedflow
  module Plugin
    # Plugin configuration
    class Configuration
      CONFIG_KEY = '_config'.freeze

      # Initialize.
      #
      # arguments   - Hash of arguments.
      # plugin_name - Name of plugin.
      #
      # Examples
      #
      #    Configuration.new({}, 'plugin')
      #    # => <Speedflow::Plugin::Configuration>
      #
      # Returns nothing.
      def initialize(arguments, plugin_name)
        @arguments   = arguments
        @plugin_name = plugin_name
      end

      # Public: All config.
      #
      # Returns Hash of all config.
      def all_config
        @arguments.key?(CONFIG_KEY) ? @arguments[CONFIG_KEY] : {}
      end

      # Public: All input.
      #
      # Returns Hash of all input.
      def all_input
        @arguments.select { |k, _| k != CONFIG_KEY }
      end

      # Public: Get a specific config.
      #
      # key           - Key of config.
      # default_value - Default value.
      #
      # Returns value of config key.
      def by_config(key, default_value = '')
        config = {}
        if @arguments.key?(CONFIG_KEY)
          if @arguments[CONFIG_KEY].key?(@plugin_name)
            config = @arguments[CONFIG_KEY][@plugin_name]
          end
        end
        config.key?(key) ? config[key].to_s : default_value.to_s
      end

      # Public: Get multiple config.
      #
      # keys_and_defaults - Hash of keys and defaults. { 'key' => 'default' }
      #
      # Returns Hash with values or defaults.
      def by_multiple_config(keys_and_defaults)
        keys_and_defaults.each { |k, v| keys_and_defaults[k] = by_config(k, v) }
      end

      # Public: Get an input config by key.
      #
      # key           - Key of config.
      # default_value - Default value.
      #
      # Returns value of input config key.
      def by_input(key, default_value = '')
        if @arguments.key?(key)
          @arguments[key]['value'].to_s
        else
          default_value.to_s
        end
      end

      # Public: Get an input config by key but required.
      #
      # key           - Key of config.
      # default_value - Default value.
      #
      # Returns value of input config key or Exception.
      def by_required_input(key, default_value = '')
        value = by_input(key, default_value)

        if by_input(key).empty?
          raise ConfigurationInputRequire, "Required value for '#{key}'."
        end

        value
      end
    end

    # Input exception
    class ConfigurationInputRequire < RuntimeError
    end
  end
end
