module Speedflow
  # Plugin
  module Plugin
    # Used to manage the plugins
    class Manager
      include ActiveSupport::Inflector
      # TODO: Remove underground prompt.
      include Message

      PLUGIN_BASE = 'speedflow-plugin-'.freeze

      # @return [Speedflow::Plugin::Configuration] Plugin configuration object.
      attr_writer :config

      # @return [Speedflow::Plugin::Prompt] Plugin prompt object.
      attr_writer :prompt

      # Public: Constructor
      #
      # plugins - Array of plugins
      #
      # Examples
      #
      #    Manager.new([])
      #    # => <Speedflow::Plugin::Manager>
      #
      # Returns an Arrays of plugins.
      def initialize(plugins = [])
        @plugins = plugins || []
        load_plugins
      end

      # Public: Load plugins
      #
      # Returns nothing.
      def load_plugins
        @plugins.each { |plugin| require_plugin(plugin) }
      end

      # Public: Require a plugins (from Gem)
      #
      # Returns nothing.
      def require_plugin(plugin)
        require plugin.downcase.prepend(PLUGIN_BASE)
      rescue LoadError
        message = "Unable to load plugin '#{plugin}'.\n"
        message << "Help: `gem install #{PLUGIN_BASE}#{plugin}`"
        raise PluginNotFound, message
      end

      # Call action from flow step.
      #
      # step - Hash from flow. {plugin: '', action: '', arguments: ''}.
      #
      # Returns nothing.
      def action_from_step(step)
        plugin = plugin(step['plugin'], step['arguments'])
        action(plugin, step['action'])
      end

      # Call plugin.
      #
      # plugin_name - Plugin name.
      # arguments   - List of arguments.
      #
      # Returns <Speedflow::Plugin::Abstract>.
      def plugin(plugin_name, arguments)
        configuration = config.new(arguments, plugin_name)
        plugin_object(plugin_name).new(configuration, prompt.new)
      end

      # Call action.
      #
      # plugin_name - Plugin name.
      # action_name - Action name.
      # arguments   - List of arguments.
      #
      # Returns Hash of action output.
      def action(plugin, action_name)
        plugin_name = plugin.class.name.split('::')[-2].downcase
        action_name = action_name.prepend('action_').underscore

        unless plugin.respond_to?(action_name)
          message = "Unable to call action: #{plugin_name}.#{action_name}"
          raise PluginActionNotFound, message
        end

        # TODO: Remove underground prompt.
        success "Run action '#{action_name}' of '#{plugin_name}' plugin."

        plugin.send action_name
      end

      # Public: Get plugin object from plugin name
      #
      # plugin_name - Plugin name.
      #
      # TODO: Create Utils::name_to_object method.
      #
      # Returns Object.
      def plugin_object(plugin_name)
        plugin_name = plugin_name.downcase.capitalize

        "Speedflow::Plugin::#{plugin_name}::Plugin".constantize
      end

      # Public: Plugin configuration object.
      #
      # Returns Speedflow::Plugin::Configuration.
      def config
        @config ||= Configuration
      end

      # Public: Plugin prompt object.
      #
      # Returns Speedflow::Plugin::Prompt.
      def prompt
        @prompt ||= ::TTY::Prompt
      end
    end

    # Plugin exception
    class PluginNotFound < RuntimeError
    end

    # Plugin action exception
    class PluginActionNotFound < RuntimeError
    end
  end
end
